@@ -1,10 +1,12 @@
 # SPDX-License-Identifier: BSD-3-Clause
-# Copyright (c) 2022 Scipp contributors (https://github.com/scipp)
+# Copyright (c) 2023 Scipp contributors (https://github.com/scipp)
 # @author Simon Heybrock
 from typing import Dict, Union
+
 import scipp as sc
-from .nxobject import NXobject, ScippIndex
+
 from ._common import to_plain_index
+from .nxobject import NXobject, ScippIndex
 
 
 class Leaf(NXobject):
@@ -17,7 +19,7 @@ class Leaf(NXobject):
         index = to_plain_index([], select)
         if index != tuple():
             raise ValueError(f"Cannot select slice when loading {type(self).__name__}")
-        content = {}
+        content = sc.DataGroup()
         for key, obj in self.items():
             if key == 'depends_on' or isinstance(obj, NXtransformations):
                 continue
